% Written for the empirical power function plots in the main paper
% Plot only BGMM and BMM estimators
clear all;
close all;
date_string = '122019';     % date used in the filename of saved simulation results
date_string_out = '122019'; % date to be used in the filename of output figures
fdir = '.';

% BMM and BGMM
% optionset = [3, 4.2];  % estimators assuming homoskedasticity
optionset = [3.1, 5.2];  % estimators assuming heteroskedasticity
flag_NormalErr = 1; % =1: Gaussian errors; =0: Chi2 errors
flag_HetVar = 1;  % =1: heteroskedasticity; =0: homoskedasticity

rho = 0.5;
N = 100;
deltaset = [0,0.5,0.75,0.95];

flag_plot_beta = 1;   % =1: plot for beta; =0: plot for rho
flag_SARX = 1;   % =1: SARX model
beta = 1;  % true value of beta
nsimul = 2000;
if flag_NormalErr==1
    err_string = 'normal';
elseif flag_NormalErr==0
    err_string = 'chi2';
end
if flag_HetVar==1
    err_string = [err_string,'_HetVar'];
end
if flag_SARX==1
    model_string = 'SARX';
elseif flag_SARX==0
    model_string = 'SAR';
end
lendel = length(deltaset);
lenoption = length(optionset);

Ngrid = 100;
siz_line = 0.05*ones(1,Ngrid);
pow_rho_grid_3d = zeros(1,Ngrid,lenoption);
pow_beta_grid_3d = zeros(1,Ngrid,lenoption);

true_rho_yline = linspace(0,0.02,Ngrid);
true_rho_xpos = rho*ones(1,Ngrid);
true_beta_yline = linspace(0,0.02,Ngrid);
true_beta_xpos = beta*ones(1,Ngrid);

hfig = figure(1); clf;
for del = 1:lendel
    for opti = 1:lenoption
        delta = deltaset(del);
        option = optionset(opti);

        if rho==0.5
            if N==100
                    rho_grid = linspace(0,0.99,Ngrid);
                    xtick = [0:0.1:1];
                    beta_grid = linspace(0,2,Ngrid);    % adding beta_grid for N=100
                    beta_xtick = [0:0.2:2];
            elseif N==300
                    rho_grid = linspace(0,0.99,Ngrid);
                    xtick = [0:0.1:1];
                    beta_grid = linspace(0,2,Ngrid);    % adding beta_grid for N=300
                    beta_xtick = [0:0.2:2];
            elseif N==500
                if delta<0.95
                    rho_grid = linspace(0.3,0.7,Ngrid);
                    xtick = [0.3:0.1:0.7];
                else
                    rho_grid = linspace(0.1,0.8,Ngrid);
                    xtick = [0.1:0.1:0.8];  
                end
            elseif N==1000
                if delta<0.95
                    rho_grid = linspace(0.35,0.65,Ngrid);
                    xtick = [0.35:0.05:0.65];
                else
                    rho_grid = linspace(0.25,0.75,Ngrid);
                    xtick = [0.25:0.1:0.75]; 
                end
            end
        elseif rho==0.75
            if N==100
                if delta<0.95
                    rho_grid = linspace(0.35,0.99,Ngrid);
                    xtick = [0.35,0.45,0.55,0.65,0.75,0.85,0.95];
                else
                    rho_grid = linspace(0.25,0.99,Ngrid);
                    xtick = [0.25:0.1:0.95];
                end
            elseif N==300
                if delta<0.95
                    rho_grid = linspace(0.55,0.9,Ngrid);
                    xtick = [0.55,0.6,0.65,0.7,0.75,0.8,0.85,0.9];
                else
                    rho_grid = linspace(0.4,0.95,Ngrid);
                    xtick = [0.4:0.05:0.95];
                end
            elseif N==500
                if delta<0.95
                    rho_grid = linspace(0.6,0.9,Ngrid);
                    xtick = [0.6,0.65,0.7,0.75,0.8,0.85,0.9];
                else
                    rho_grid = linspace(0.5,0.9,Ngrid);
                    xtick = [0.5,0.55,0.6,0.65,0.7,0.75,0.8,0.85,0.9]; 
                end
            elseif N==1000
                if delta<0.95
                    rho_grid = linspace(0.65,0.85,Ngrid);
                    xtick = [0.65:0.05:0.85];
                else
                    rho_grid = linspace(0.55,0.9,Ngrid);
                    xtick = [0.55:0.05:0.9];
                end
            end
            elseif rho==0.95
                if N==100
                    if delta<0.95
                        rho_grid = linspace(0.75,0.99,Ngrid);
                        xtick = [0.75:0.05:0.99];
                    else
                        rho_grid = linspace(0.55,0.99,Ngrid);
                        xtick = [0.55:0.1:0.99];
                    end
                elseif N==300
                    if delta<0.95
                        rho_grid = linspace(0.87,0.99,Ngrid);
                        xtick = [0.87:0.02:0.99];
                    else
                        rho_grid = linspace(0.75,0.99,Ngrid);
                        xtick = [0.75:0.05:0.99];    
                    end
                elseif N==500
                    if delta<0.95
                        rho_grid = linspace(0.89,0.99,Ngrid);
                        xtick = [0.89:0.02:0.99]; 
                    else
                        rho_grid = linspace(0.83,0.99,Ngrid);
                        xtick = [0.83:0.02:0.99];      
                    end
                elseif N==1000
                    if delta<0.95
                        rho_grid = linspace(0.91,0.99,Ngrid);
                        xtick = [0.91:0.02:0.99]; 
                    else
                        rho_grid = linspace(0.87,0.99,Ngrid);
                        xtick = [0.87:0.02:0.99];  
                    end
                end
        end

        rho_gridBIG = repmat(rho_grid,nsimul,1);
        beta_gridBIG = repmat(beta_grid,nsimul,1);
        
        fpath = [fdir,'\results_SARX_',date_string,'\'];
        fname = [fpath, 'results_',date_string,'_',model_string,'_',err_string,'_',num2str(option),'_',...
            num2str(rho),'_',num2str(N),'_',num2str(delta),'.mat'];                     
        load(fname,'parmhatBIG','seBIG');   
                
        pow_rho_grid = mean(abs(bsxfun(@rdivide,bsxfun(@minus,parmhatBIG(:,1),rho_gridBIG),seBIG(:,1)))>1.96);
        pow_rho_grid_3d(:,:,opti) = pow_rho_grid;
        
        pow_beta_grid = mean(abs(bsxfun(@rdivide,bsxfun(@minus,parmhatBIG(:,3),beta_gridBIG),seBIG(:,3)))>1.96);
        pow_beta_grid_3d(:,:,opti) = pow_beta_grid;            
        
    end 
    if flag_plot_beta~=1
    subh = subplot(2,2,del);   
    marker1 = pow_rho_grid_3d(:,:,1);
    co = get(gca,'ColorOrder');  
    myco = [co(1:2,:);co(1,:);co(1,:);[0.25,0.25,0.25]];
    set(gca, 'ColorOrder', myco, 'NextPlot', 'replacechildren');
    % NOTE: the first line is a dummy for legend       
    plot(rho_grid(1),marker1(1),'-o',...
        rho_grid,pow_rho_grid_3d(:,:,2),'--',...
        rho_grid(1:10:end),marker1(1:10:end),'o',... 
        rho_grid,pow_rho_grid_3d(:,:,1),...,
        rho_grid,siz_line,':','markers',5,'Linewidth',1.1);
    hold on;
    plot(true_rho_xpos, true_rho_yline,'k','Linewidth',2);  % mark the true value of rho
    hold off;
    FontSize = 11;
    str_title = strcat('\delta=',num2str(delta));
    title(str_title,'FontWeight','Normal','FontSize',FontSize);         

    ax = gca;
    set(ax,'YTick',[0 0.05 0.2 0.4 0.6 0.8 1])
    set(ax,'XTick',xtick);
    xlim([rho_grid(1),rho_grid(end)]);
    xlabel('\rho');
    
    elseif flag_plot_beta==1
        subh = subplot(2,2,del);   
        marker1 = pow_beta_grid_3d(:,:,1);
        co = get(gca,'ColorOrder');
        myco = [co(1:2,:);co(1,:);co(1,:);[0.25,0.25,0.25]];
        set(gca, 'ColorOrder', myco, 'NextPlot', 'replacechildren');
        % NOTE: the first line is a dummy for legend       
        plot(beta_grid(1),marker1(1),'-o',...
            beta_grid,pow_beta_grid_3d(:,:,2),'--',...
            beta_grid(1:10:end),marker1(1:10:end),'o',... 
            beta_grid,pow_beta_grid_3d(:,:,1),...,
            beta_grid,siz_line,':','markers',5,'Linewidth',1.1);
        hold on;
        plot(true_beta_xpos, true_beta_yline,'k','Linewidth',2);  % mark the true value of rho
        hold off;
        FontSize = 11;
        str_title = strcat('\delta=',num2str(delta));
        title(str_title,'FontWeight','Normal','FontSize',FontSize);         

        ax = gca;
        set(ax,'YTick',[0 0.05 0.2 0.4 0.6 0.8 1])
        set(ax,'XTick',beta_xtick);
        xlim([beta_grid(1),beta_grid(end)]);
        xlabel('\beta');
    end
    
end
leg = legend('BMM','GMM');
legend boxoff
set(leg,'Orientation','horizontal','Position',[0.36,0,0.3,0.1]);

figpath = [fdir,'\figs\figs_',date_string_out,'\'];
if ~exist(figpath, 'dir')
    mkdir(figpath);
end
if flag_plot_beta~=1
    figname = [figpath,'fig_power_',model_string,'_',err_string,'_rho',num2str(rho*100),'_',num2str(N),'.eps'];
elseif flag_plot_beta==1
    figname = [figpath,'fig_power_',model_string,'_',err_string,'_rho',num2str(rho*100),'_',num2str(N),'_beta.eps'];
end
figure_width  = 11*2;
figure_height = 10*2;
set(gcf, 'units', 'centimeters', 'pos', [0 0 figure_width figure_height])
set(gcf, 'PaperPositionMode', 'auto')
set(gcf,'renderer','painters')
print(gcf, '-r300','-depsc2',figname);
