% Written for the empirical power function plots in the supplement
% Plot BMM, BGMM and ML estimators
clear all;
close all;
date_string = '122019';     % date used in the filename of saved simulation results
date_string_out = '122019'; % date to be used in the filename of output figures
fdir = '.';

% BMM, BGMM and ML
optionset = [3, 4.2, 1]; % Estimation Option, see main_MC.m for details
rho = 0.5;
Nset = [100, 300, 500, 1000];
delta_choice_set = [1,2,3];

for Ni = 1:length(Nset)
    N = Nset(Ni);
for delta_choicei = 1:length(delta_choice_set)
    delta_choice = delta_choice_set(delta_choicei);
    
if delta_choice ==1
    deltaset = [0,0.25];
    delta_string = 'deltaL';
elseif delta_choice ==2
    deltaset = [0.5,0.75];
    delta_string = 'deltaM';
elseif delta_choice ==3
    deltaset = [0.95,1];    
    delta_string = 'deltaH';
end
    
flag_NormalErr = 1;
flag_SARX = 1;
nu = 5;  
nsimul = 2000;
if flag_NormalErr==1
    err_string = 'normal';
elseif flag_NormalErr==0
    err_string = 'chi2';
elseif flag_NormalErr==2
    err_string = strcat('t-',num2str(nu));
end    
if flag_SARX==1
    model_string = 'SARX';
elseif flag_SARX==0
    model_string = 'SAR';
end
lendel = length(deltaset);
lenoption = length(optionset);

Ngrid = 100;
true_rho_yline = linspace(0,0.02,Ngrid);
true_rho_xpos = rho*ones(1,Ngrid);

if rho==0.2
    rho_grid_b = rho-0.6; % beginning
    rho_grid_e = rho+0.6; % end
    rho_grid = linspace(rho_grid_b,rho_grid_e,Ngrid);
    xtick = [rho_grid_b:0.1:rho_grid_e];
elseif rho==0.5
    rho_grid_b = rho-0.5; 
    rho_grid_e = rho+0.5;
    rho_grid = linspace(rho_grid_b,rho_grid_e,Ngrid);
    xtick = [rho_grid_b:0.1:rho_grid_e];
elseif rho==0.75
    rho_grid_b = rho-0.25; 
    rho_grid_e = rho+0.25; 
    if N==100
        rho_grid_b = rho-0.35; 
    end
    rho_grid = linspace(rho_grid_b,rho_grid_e,Ngrid);
    xtick = [rho_grid_b:0.05:rho_grid_e];
end


siz_line = 0.05*ones(1,length(rho_grid));
rho_gridBIG = repmat(rho_grid,nsimul,1);
pow_rho_grid_3d = zeros(1,length(rho_grid),lenoption);


hfig = figure(1); clf;
for del = 1:lendel
    for opti = 1:lenoption
        delta = deltaset(del);
        option = optionset(opti);

        fpath =['.\results_SARX_',date_string,'\']; 
                    fname =  [fpath, 'results_',date_string,'_',model_string,'_',err_string,'_',num2str(option),'_',...
                    num2str(rho),'_',num2str(N),'_',num2str(delta),'.mat'];
        load(fname,'parmhatBIG','seBIG');
        pow_rho_grid = mean(abs(bsxfun(@rdivide,bsxfun(@minus,parmhatBIG(:,1),rho_gridBIG),seBIG(:,1)))>1.96);
        pow_rho_grid_3d(:,:,opti) = pow_rho_grid;
    end 
    subh = subplot(2,1,del);   
    marker1 = pow_rho_grid_3d(:,:,1);
    marker3 = pow_rho_grid_3d(:,:,3); 
    co = get(gca,'ColorOrder');
    myco = [co(1:2,:);co(5,:);co(5,:);co(5,:);co(1,:);co(1,:);[0.25,0.25,0.25]];
    set(gca, 'ColorOrder', myco, 'NextPlot', 'replacechildren');
    % NOTE: the first line is a dummy for legend        
    plot(rho_grid(1),marker1(1),'-o',...
        rho_grid,pow_rho_grid_3d(:,:,2),'--',...
        rho_grid(5), marker3(5),':*',...
        rho_grid(5:10:end),marker3(5:10:end),'*',...
        rho_grid,pow_rho_grid_3d(:,:,3),':',...
        rho_grid(1:10:end),marker1(1:10:end),'o',... 
        rho_grid,pow_rho_grid_3d(:,:,1),...,
        rho_grid,siz_line,':','markers',5,'Linewidth',1.1);     
    hold on;
    plot(true_rho_xpos, true_rho_yline,'k','Linewidth',2);  % mark the ture value of rho
    hold off;
    FontSize = 11;
    str_title = strcat('n=',num2str(N),', \delta=',num2str(delta));
    title(str_title,'FontWeight','Normal','FontSize',FontSize);         

    ax = gca;
    set(ax,'YTick',[0 0.05 0.2 0.4 0.6 0.8 1])
    set(ax,'XTick',xtick);
    xlim([rho_grid(1),rho_grid(end)]);
    xlabel('\rho');
end
leg = legend('BMM','GMM','ML');
legend boxoff
set(leg,'Orientation','horizontal','Position',[0.22,0,0.5,0.1]);

figpath = [fdir,'\figs\figs_',date_string_out,'\'];
if ~exist(figpath, 'dir')
    mkdir(figpath);
end
figname = [figpath,'fig_power_',model_string,'_',err_string,'_rho',num2str(rho*100),'_',num2str(N),'_',delta_string,'.eps'];
figure_width  = 6*2;
figure_height = 11*2;
set(gcf, 'units', 'centimeters', 'pos', [0 0 figure_width figure_height])
set(gcf, 'PaperPositionMode', 'auto')
set(gcf,'renderer','painters')
print(gcf, '-r300','-depsc2',figname);

end
end  