function results = BMM_SARX(Y,X,Ystar,W,flag_HetVar,info)
% Computes BMM estimates

[N,k] = size(X);
% set defaults
options = optimoptions(@lsqnonlin,'Algorithm','trust-region-reflective','Display','off'); 
options.MaxFunEvals = 1e10;
options.MaxIter = 1e10;
% options.Display = 'iter';
% options.Display = 'iter-detailed'; 
options.TolFun = 1e-6;
Mb = 1000;           
if flag_HetVar==0     % homoskedasticity
    parm_lb = [-1;-Mb*ones(k,1);0];
    parm_ub = [1;Mb*ones(k+1,1)];
    parm0 = [0;zeros(k,1);1];
elseif flag_HetVar==1  % heteroskedasticity
    parm0 = [0;zeros(k,1)]; 
    parm_lb = [-1;-Mb*ones(k,1)]; 
    parm_ub = [1;Mb*ones(k,1)];        
end

if ~isstruct(info)
    error('Must supply the options as a structure variable');
end
fields = fieldnames(info);
nf = length(fields);
if nf > 0
    for i=1:nf
        if strcmp(fields{i},'parm_lb')
            parm_lb = info.parm_lb;  
        elseif strcmp(fields{i},'parm_ub')
            parm_ub = info.parm_ub;  
        elseif strcmp(fields{i},'parm0')
            parm0 = info.parm0;
        end
    end
end
if length(parm0)~=(k+2-flag_HetVar)
    error('Wrong parameter dimension!');
end

if flag_HetVar==0
    [parmhat,resnorm,residual,exitflag,output] = lsqnonlin(@BMM_moment_SARX_iid,parm0,parm_lb,parm_ub,options,Y,X,Ystar,W,N);
    sige2hat = parmhat(k+2);
    if sige2hat<=0
        error('negative sige2hat!');
    end
    results.sige2 = sige2hat;
elseif flag_HetVar==1
    [parmhat,resnorm,residual,exitflag,output] = lsqnonlin(@BMM_moment_SARX_het,parm0,parm_lb,parm_ub,options,Y,X,Ystar,W,N);
end
if exitflag~=1 && exitflag~=3
    fprintf('\n iter=%d exitflag=%d \n',iter,exitflag);
    pause;
end
rhohat = parmhat(1);
betahat = parmhat(2:k+1);
eps_bound = 1e-3;
if abs(rhohat-parm_lb(1))<eps_bound
   rhohat = parm_lb(1)+eps_bound;   
   warning('boundary solution!')
elseif abs(rhohat-parm_ub(1))<eps_bound 
   rhohat = parm_ub(1)-eps_bound;
   warning('boundary solution!')
end   

% variance
Shat = speye(N)-rhohat*W;
Ghat = W/Shat;
GXb = Ghat * X *betahat;
GXb2 = GXb'*GXb/N;
ehatN = Y - rhohat*Ystar - X*betahat;
if flag_HetVar==0
    Mx = eye(N) - X/(X'*X)*X';
    % H matrix
    H = zeros(k+1,k+1);
    trGhat = trace(Ghat);
    Gterm = sige2hat*(sum(sum(Ghat.*Ghat)+sum(Ghat.*Ghat'))/N - sum(sum(Ghat.*Mx))*trGhat*2/N^2);
    H(1,1) = GXb2 + Gterm;
    H(2:k+1,1) = X'*GXb/N;
    H(1,2:k+1) = H(2:k+1,1)';
    Sigxx =  X'*X/N;
    H(2:k+1,2:k+1) = Sigxx;
    % Var(g) matrix
    C = zeros(k+1,k+1);
    C(2:k+1,1) = sige2hat*H(2:k+1,1);
    C(1,2:k+1) = C(2:k+1,1)';
    C(2:k+1,2:k+1) = sige2hat*Sigxx;
    D = Ghat - Mx*(trGhat/N);
    Dterm = sige2hat^2*sum(sum(D.*D)+sum(D.*D'))/N;
    gamma2hat = sum(ehatN.^4)/N - 3*sige2hat^2; 
    diagD = diag(D);
    non_normal_term1 = gamma2hat*sum(diagD.*diagD)/N;
    mu3 = sum(ehatN.^3)/N; 
    non_normal_term2 = 2*mu3*sum(diagD.*GXb)/N;
    non_normal_term = non_normal_term1 + non_normal_term2;  
    C(1,1) = sige2hat*GXb2 + non_normal_term + Dterm;
elseif flag_HetVar==1   
    e2hatN = ehatN.^2;
    SigGxb = e2hatN.*GXb;
   % H matrix
    H = zeros(k+1,k+1);
    diagG = diag(Ghat);
    Gterm = sum( diag((Ghat^2 + Ghat'*Ghat - diagG.*Ghat*2)) .* e2hatN)/N;
    H(1,1) = GXb2 + Gterm;
    H(2:k+1,1) = X'*GXb/N;
    H(1,2:k+1) = H(2:k+1,1)';
    Sigxx =  X'*X/N;
    H(2:k+1,2:k+1) = Sigxx;
    % Var(g) matrix
    C = zeros(k+1,k+1);
    C(2:k+1,1) = X'*SigGxb/N;      
    C(1,2:k+1) = C(2:k+1,1)';
    C(2:k+1,2:k+1) =  X'*(e2hatN.*X)/N;              
    D = Ghat;
    D(1:N+1:end) = 0; 
    SigD = e2hatN.*D;        
    Dterm = trace(SigD * (SigD + e2hatN.*D'))/N;
    C(1,1) = GXb'*SigGxb/N + Dterm;    
end
covmtx = H\C/H/N;
se = sqrt(diag(covmtx));
se_rhohat = se(1);
se_betahat = se(2:k+1);    

results.rho = rhohat;
results.beta = betahat;
results.se_rho = se_rhohat;
results.se_beta = se_betahat;
end